#ifndef BARREDEMENU_H
#define BARREDEMENU_H

#include <qwidget.h>
#include <qmenubar.h>
#include <qtextbrowser.h>

#include <noynoyau.h>
#include <noyobservateur.h>
#include <noyelementdemodelisation.h>

#include <ihmvisiteursupprimer.h>


/** Cette classe represente une barre de menu de l'interface.
 * @author F.Duchaussoy, N.Longuet, G.Prevot
 */

class BarreDeMenu : public QMenuBar, public Observateur
{
	Q_OBJECT

	public:

		/** Constructeur.
		 * \param Noyau *n : Represente le noyau associ au projet.
		 * \param QWidget *parent : Represente le widget parent de la barre de menu (Par defaut 0).
		 * \param const char *name : Represente le nom que l'on veut donner  notre barre de menu (Par defaut 0).
		 */
		BarreDeMenu(Noyau * n, QWidget *parent=0, const char *name=0);

		//! Destructeur
		virtual ~BarreDeMenu() {}


		/** Cette methode averti la barre de menu qu'il y a eu un changement dans le noyau.
		 * Elle provoque le raffraichissement de la barre de menu.
		 */
		void avertir();

		/** Permet de recuperer le noyau qu'coute cet barre de menu.
		 * \return Une reference sur le noyau qu'coute cet barre de menu.
		 */ 
		Noyau * getNoyau() {return noyau;}


		/** Permet de recuperer une reference sur le QPopMenu projet de la barre de menu.
		 * \return Une reference sur le QPopupMenu projet de la barre de menu.
		 */
		QPopupMenu *getProjet() { return projet; }

		/** Permet de recuperer une reference sur le QPopMenu edition de la barre de menu.
		 * \return Une reference sur le QPopupMenu edition de la barre de menu.
		 */
		QPopupMenu *getEdition() { return edition; }

		/** Permet de recuperer une reference sur le QPopMenu outils de la barre de menu.
		 * \return Une reference sur le QPopupMenu outils de la barre de menu.
		 */
		QPopupMenu *getOutils() { return outils; }

		/** Permet de recuperer une reference sur le QPopMenu ajouter de la barre de menu.
		 * \return Une reference sur le QPopupMenu ajouter de la barre de menu.
		 */
		QPopupMenu *getAjouter() { return ajouter; }

		/** Permet de recuperer une reference sur le QPopMenu exporter de la barre de menu.
		 * \return Une reference sur le QPopupMenu exporter de la barre de menu.
		 */
		QPopupMenu *getExporter() { return exporter; }

		/** Permet de recuperer une reference sur le QPopMenu parser de la barre de menu.
		 * \return Une reference sur le QPopupMenu parser de la barre de menu.
		 */
		QPopupMenu *getParser() { return parser; }

		int ouvrirID, nouveauID, enregistrerID, informationsID, cloreID, quitterID  ;
		int couperID, copierID, collerID, supprimerID, preferenceID ;
		int ajoutVueID, ajoutPaquetageID, ajoutCommentaireID, ajoutClasseID, ajoutMethodeID, ajoutAttributID, ajoutRelationID ;
		int exportCppID, exportJavaID, importCppID, importJavaID ;

		inline void setAjoutOK(bool val) {ajoutOK = val;}
		inline Noeud *getNoeudEnMemoire() {return noeudEnMemoire;}
		inline void setNoeudEnMemoire(Noeud *n) {noeudEnMemoire = n;}

	private:
		QPopupMenu *projet, *edition, *outils, *ajouter, *exporter, *parser;
		Noyau * noyau;
		VisiteurSupprimer * visiteurSupprimer ;
		Noeud *noeudEnMemoire;
		bool couperEnMemoire, ajoutOK;

	public slots:
		// Projet
		//! Ce slot permet de lancer la procedure d'ouverture d'un projet.
		void ouvrir();
		//! Ce slot permet de lancer la procedure de creation d'un projet.
		void nouveau();
		//! Ce slot permet de lancer la procedure d'enregistrement d'un projet.
		void enregistrer();
		//! Ce slot permet de lancer la cloture du projet
		void fermer();
		//! Ce slot est appel lors de la selection du menu Projet/informations.
		void informations();
		//! Ce slot quitte le programme.
		void quitter();

		// Edition
		/** Ce slot "coupe" le noeud courant. C'est--dire qu'une reference sur ce noeud sera
		 * garde en memoire. Le noeud sera effac si le slot coller est activ. Sinon,  la prochaine
		 * operation de couper ou copier, la reference du noeud en memoire sera perdu.
		 */
		void couper();
		/** Ce slot garde un copie du noeud courant en memoire pour le dupliquer
		 * lors de l'activation du slot coller. La reference du noeud courant est perdue
		 * lors d'une nouvelle activation du slot couper ou copier.
		 */
		void copier();
		//! Ce slot "colle" le noeud gard en memoire par les slot couper et copier  l'endroit voulu.
		void coller();
		//! Ce slot supprime le noeud courant.
		void supprimer();
		//! Ce slot lance la procedure de configuration des preferences.
		void preferences();
	
		// Outils/Ajouter
		//! Ce slot lance la procedure d'ajout d'une vue.
		void vue();
		//! Ce slot lance la procedure d'ajout d'un paquetage.
		void paquetage();
		//! Ce slot lance la procedure d'ajout d'un commentaire.
		void commentaire();
		//! Ce slot lance la procedure d'ajout d'une classe.
		void classe();
		//! Ce slot lance la procedure d'ajout d'une methode.
		void methode();
		//! Ce slot lance la procedure d'ajout d'un attribut.
		void attribut();
		//! Ce slot lance la procedure d'ajout d'une relation.
		void relation();

		// Outils/Exporter
		//! Ce slot lance la procedure d'exportation en C++.
		void exportCpp();
		//! Ce slot lance la procedure d'exportation en Java.
		void exportJava();

		// Outils/Parser
		//! Ce slot lance la procedure d'importation de code C++.
		void importCpp();
		//! Ce slot lance la procedure d'importation de cod Java.
		void importJava();

		// Aide
		//! Ce slot lance un mini-navigateur pour parcourir l'aide.
		void aide();
		//! Ce slot fait apparaitre la fentre  propos du logiciel.
		void aPropos();

		// Exterieur a la barre de menu
		/** Ce slot tente d'ajouter le noeud courant dans une vue.
		 * \param int vueID : correspond  l'identifiant de la vue dans laquelle on veut ajouter le noeud.
		 */
		void ajouterDansVue( int vueID );

		// Exterieur a la barre de menu
		/** Ce slot sert  ajouter une relationCommentaire en slectionnant l'entre "cibleID" lors d'un
		 * clic-droit sur un commentaire.
		 * \param int cibleID : correspond  l'identifiant de l'objet auquel on veut lier le neud.
		 */
		void lierCommentaire( int cibleID );

};

#endif

